@@ -98,7 +98,7 @@ func Walk(v Visitor, node Node) {
 		}
 
 	// Expressions
-	case *BadExpr, *Ident:
+	case *BadExpr, *Ident, *NumberUnitLit:
 		// nothing to do
 
 	case *BasicLit:
@@ -110,6 +110,16 @@ func Walk(v Visitor, node Node) {
 			}
 		}
 
+	case *DomainTextLit:
+		Walk(v, n.Domain)
+		if n.Extra != nil { // Go+ extended
+			for _, part := range n.Extra.Parts {
+				if e, ok := part.(Expr); ok {
+					Walk(v, e)
+				}
+			}
+		}
+
 	case *Ellipsis:
 		if n.Elt != nil {
 			Walk(v, n.Elt)
@@ -136,6 +146,10 @@ func Walk(v Visitor, node Node) {
 		Walk(v, n.X)
 		Walk(v, n.Index)
 
+	case *IndexListExpr:
+		Walk(v, n.X)
+		walkExprList(v, n.Indices)
+
 	case *SliceExpr:
 		Walk(v, n.X)
 		if n.Low != nil {
@@ -219,7 +233,9 @@ func Walk(v Visitor, node Node) {
 
 	case *SendStmt:
 		Walk(v, n.Chan)
-		Walk(v, n.Value)
+		for _, val := range n.Values {
+			Walk(v, val)
+		}
 
 	case *IncDecStmt:
 		Walk(v, n.X)
@@ -452,6 +468,9 @@ func Walk(v Visitor, node Node) {
 		Walk(v, n.Name)
 		walkExprList(v, n.Funcs)
 
+	case *EnvExpr:
+		Walk(v, n.Name)
+
 	default:
 		panic(fmt.Sprintf("ast.Walk: unexpected node type %T", n))
 	}
