@@ -17,15 +17,14 @@
 package cl
 
 import (
-	"errors"
 	"go/types"
 	"log"
 	"testing"
 
 	"github.com/goplus/gogen"
-	"github.com/goplus/gogen/cpackages"
 	"github.com/goplus/gogen/packages"
 	"github.com/goplus/gop/ast"
+	"github.com/goplus/gop/parser"
 	"github.com/goplus/gop/token"
 	"github.com/goplus/mod/modfile"
 )
@@ -40,6 +39,15 @@ func getGoxConf() *gogen.Config {
 	return &gogen.Config{Fset: fset, Importer: imp}
 }
 
+func TestSimplifyPkgPath(t *testing.T) {
+	if simplifyPkgPath("c/lua") != "github.com/goplus/llgo/c/lua" {
+		t.Fatal("simplifyPkgPath: c/lua")
+	}
+	if simplifyPkgPath("cpp/std") != "github.com/goplus/llgo/cpp/std" {
+		t.Fatal("simplifyPkgPath: cpp/std")
+	}
+}
+
 func TestCompileLambdaExpr(t *testing.T) {
 	ctx := &blockCtx{
 		pkgCtx: &pkgCtx{},
@@ -92,11 +100,14 @@ func TestCompileLambda2(t *testing.T) {
 
 func TestCompileExpr(t *testing.T) {
 	defer func() {
-		if e := recover(); e != "compileExpr failed: unknown - *ast.Ellipsis\n" {
-			t.Fatal("compileExpr:", e)
+		if e := recover(); e != nil {
+			if ce := e.(*gogen.CodeError); ce.Msg != "compileExpr failed: unknown - *ast.Ellipsis" {
+				t.Fatal("compileExpr:", ce.Msg)
+			}
 		}
 	}()
-	compileExpr(nil, &ast.Ellipsis{})
+	ctx := &blockCtx{pkgCtx: &pkgCtx{}}
+	compileExpr(ctx, &ast.Ellipsis{})
 }
 
 func TestCompileStmt(t *testing.T) {
@@ -151,85 +162,6 @@ func TestErrStringLit(t *testing.T) {
 func TestErrPreloadFile(t *testing.T) {
 	pkg := gogen.NewPackage("", "foo", goxConf)
 	ctx := &blockCtx{pkgCtx: &pkgCtx{}}
-	t.Run("overloadName", func(t *testing.T) {
-		defer func() {
-			if e := recover(); e == nil || e != "TODO - can't overload operator ++\n" {
-				t.Fatal("TestErrPreloadFile:", e)
-			}
-		}()
-		overloadName(&ast.Ident{}, "++", true)
-	})
-	t.Run("checkOverloadFunc", func(t *testing.T) {
-		defer func() {
-			if e := recover(); e == nil || e != "TODO - cl.preloadFile OverloadFuncDecl: checkOverloadFunc\n" {
-				t.Fatal("TestErrPreloadFile:", e)
-			}
-		}()
-		checkOverloadFunc(&ast.OverloadFuncDecl{
-			Recv: &ast.FieldList{},
-		})
-	})
-	t.Run("checkOverloadMethod", func(t *testing.T) {
-		defer func() {
-			if e := recover(); e == nil || e != "TODO - cl.preloadFile OverloadFuncDecl: checkOverloadMethod\n" {
-				t.Fatal("TestErrPreloadFile:", e)
-			}
-		}()
-		checkOverloadMethod(&ast.OverloadFuncDecl{})
-	})
-	t.Run("checkOverloadMethodRecvType1", func(t *testing.T) {
-		defer func() {
-			if e := recover(); e == nil || e != "TODO - checkOverloadMethodRecvType: bar\n" {
-				t.Fatal("TestErrPreloadFile:", e)
-			}
-		}()
-		checkOverloadMethodRecvType(&ast.Ident{Name: "foo"}, &ast.Ident{Name: "bar"})
-	})
-	t.Run("checkOverloadMethodRecvType2", func(t *testing.T) {
-		defer func() {
-			if e := recover(); e == nil || e != "TODO - checkOverloadMethodRecvType: &{0 INT 123 <nil>}\n" {
-				t.Fatal("TestErrPreloadFile:", e)
-			}
-		}()
-		expr := &ast.BasicLit{Kind: token.INT, Value: "123"}
-		checkOverloadMethodRecvType(&ast.Ident{Name: "foo"}, expr)
-	})
-	t.Run("OverloadFuncDecl: invalid recv", func(t *testing.T) {
-		defer func() {
-			if e := recover(); e == nil || e != "TODO - cl.preloadFile OverloadFuncDecl: invalid recv\n" {
-				t.Fatal("TestErrPreloadFile:", e)
-			}
-		}()
-		decls := []ast.Decl{
-			&ast.OverloadFuncDecl{
-				Name: &ast.Ident{Name: "add"},
-				Funcs: []ast.Expr{
-					&ast.FuncLit{},
-				},
-				Recv: &ast.FieldList{List: []*ast.Field{
-					{Type: &ast.StarExpr{}},
-				}},
-			},
-		}
-		preloadFile(pkg, ctx, &ast.File{Decls: decls}, "", true)
-	})
-	t.Run("OverloadFuncDecl: unknown func", func(t *testing.T) {
-		defer func() {
-			if e := recover(); e == nil || e != "TODO - cl.preloadFile OverloadFuncDecl: unknown func - *ast.BasicLit\n" {
-				t.Fatal("TestErrPreloadFile:", e)
-			}
-		}()
-		decls := []ast.Decl{
-			&ast.OverloadFuncDecl{
-				Name: &ast.Ident{Name: "add"},
-				Funcs: []ast.Expr{
-					&ast.BasicLit{},
-				},
-				Operator: true,
-			},
-		}
-		preloadFile(pkg, ctx, &ast.File{Decls: decls}, "", true)
-	})
 	t.Run("unknown decl", func(t *testing.T) {
 		defer func() {
 			if e := recover(); e == nil || e != "TODO - cl.preloadFile: unknown decl - *ast.BadDecl\n" {
@@ -253,6 +185,17 @@ func TestErrParseTypeEmbedName(t *testing.T) {
 }
 
 func TestGmxCheckProjs(t *testing.T) {
+	_, multi := gmxCheckProjs(nil, &pkgCtx{
+		projs: map[string]*gmxProject{
+			".a": {hasMain_: true}, ".b": {hasMain_: true},
+		},
+	})
+	if !multi {
+		t.Fatal("gmxCheckProjs: not multi?")
+	}
+}
+
+func TestGmxCheckProjs2(t *testing.T) {
 	_, multi := gmxCheckProjs(nil, &pkgCtx{
 		projs: map[string]*gmxProject{
 			".a": {}, ".b": {},
@@ -300,15 +243,89 @@ func TestClassNameAndExt(t *testing.T) {
 	}
 }
 
-func TestErrMultiStarRecv(t *testing.T) {
-	defer func() {
-		if e := recover(); e == nil {
-			t.Fatal("TestErrMultiStarRecv: no panic?")
+func TestFileClassType(t *testing.T) {
+	type testData struct {
+		isClass     bool
+		isNormalGox bool
+		isProj      bool
+		fileName    string
+		classType   string
+		isTest      bool
+		found       bool
+	}
+	tests := []*testData{
+		{false, false, false, "abc.gop", "", false, false},
+		{false, false, false, "abc_test.gop", "", true, false},
+
+		{true, true, false, "abc.gox", "abc", false, true},
+		{true, true, false, "Abc.gox", "Abc", false, true},
+		{true, true, false, "abc_demo.gox", "abc", false, true},
+		{true, true, false, "Abc_demo.gox", "Abc", false, true},
+
+		{true, true, false, "main.gox", "_main", false, true},
+		{true, true, false, "main_demo.gox", "_main", false, true},
+		{true, true, false, "abc_xtest.gox", "abc", false, true},
+		{true, true, false, "main_xtest.gox", "_main", false, true},
+
+		{true, true, false, "abc_test.gox", "case_abc", true, true},
+		{true, true, false, "Abc_test.gox", "caseAbc", true, true},
+		{true, true, false, "main_test.gox", "case_main", true, true},
+
+		{true, false, false, "get.yap", "get", false, true},
+		{true, false, false, "get_p_#id.yap", "get_p_id", false, true},
+		{true, false, true, "main.yap", "AppV2", false, true},
+
+		{true, false, false, "abc_yap.gox", "abc", false, true},
+		{true, false, false, "Abc_yap.gox", "Abc", false, true},
+		{true, false, true, "main_yap.gox", "App", false, true},
+
+		{true, false, true, "abc_yap.gox", "abc", false, true},
+		{true, false, true, "Abc_yap.gox", "Abc", false, true},
+		{true, false, true, "main_yap.gox", "App", false, true},
+
+		{true, false, false, "abc_ytest.gox", "case_abc", true, true},
+		{true, false, false, "Abc_ytest.gox", "caseAbc", true, true},
+		{true, false, true, "main_ytest.gox", "App", true, true},
+	}
+	lookupClass := func(ext string) (c *Project, ok bool) {
+		switch ext {
+		case ".yap":
+			return &modfile.Project{
+				Ext: ".yap", Class: "AppV2",
+				Works:    []*modfile.Class{{Ext: ".yap", Class: "Handler"}},
+				PkgPaths: []string{"github.com/goplus/yap"}}, true
+		case "_yap.gox":
+			return &modfile.Project{
+				Ext: "_yap.gox", Class: "App",
+				PkgPaths: []string{"github.com/goplus/yap"}}, true
+		case "_ytest.gox":
+			return &modfile.Project{
+				Ext: "_ytest.gox", Class: "App",
+				Works:    []*modfile.Class{{Ext: "_ytest.gox", Class: "Case"}},
+				PkgPaths: []string{"github.com/goplus/yap/ytest", "testing"}}, true
 		}
-	}()
-	getRecvType(&ast.StarExpr{
+		return
+	}
+	for _, test := range tests {
+		_ = test.found
+		f := &ast.File{IsClass: test.isClass, IsNormalGox: test.isNormalGox, IsProj: test.isProj}
+		classType, isTest := GetFileClassType(f, test.fileName, lookupClass)
+		if isTest != test.isTest {
+			t.Fatalf("%v check classType isTest want %v, got %v.", test.fileName, test.isTest, isTest)
+		}
+		if classType != test.classType {
+			t.Fatalf("%v getClassType want %v, got %v.", test.fileName, test.classType, classType)
+		}
+	}
+}
+
+func TestErrMultiStarRecv(t *testing.T) {
+	_, _, ok := getRecvType(&ast.StarExpr{
 		X: &ast.StarExpr{},
 	})
+	if ok {
+		t.Fatal("TestErrMultiStarRecv: no error?")
+	}
 }
 
 func TestErrAssign(t *testing.T) {
@@ -544,12 +561,6 @@ func TestGetGoFile(t *testing.T) {
 	}
 }
 
-func TestC2goBase(t *testing.T) {
-	if c2goBase("") != "github.com/goplus/" {
-		t.Fatal("c2goBase failed")
-	}
-}
-
 func TestErrNewType(t *testing.T) {
 	testPanic(t, `bar redeclared in this block
 	previous declaration at <TODO>
@@ -560,25 +571,6 @@ func TestErrNewType(t *testing.T) {
 	})
 }
 
-func TestErrLoadImport(t *testing.T) {
-	testPanic(t, "-: unknownpkg not found or not a valid C package (c2go.a.pub file not found).\n", func() {
-		pkg := &pkgCtx{
-			nodeInterp: &nodeInterp{
-				fset: token.NewFileSet(),
-			},
-			cpkgs: cpackages.NewImporter(
-				&cpackages.Config{LookupPub: func(pkgPath string) (pubfile string, err error) {
-					return "", errors.New("not found")
-				}})}
-		ctx := &blockCtx{pkgCtx: pkg}
-		spec := &ast.ImportSpec{
-			Path: &ast.BasicLit{Kind: token.STRING, Value: `"C/unknownpkg"`},
-		}
-		loadImport(ctx, spec)
-		panic(ctx.errs[0].Error())
-	})
-}
-
 func TestErrCompileBasicLit(t *testing.T) {
 	testPanic(t, "compileBasicLit: invalid syntax\n", func() {
 		ctx := &blockCtx{cb: new(gogen.CodeBuilder)}
@@ -599,4 +591,16 @@ func testPanic(t *testing.T, panicMsg string, doPanic func()) {
 	})
 }
 
+func TestClassFileEnd(t *testing.T) {
+	fset := token.NewFileSet()
+	f, err := parser.ParseFile(fset, "get.yap", `json {"id": ${id} }`, parser.ParseGoPlusClass)
+	if err != nil {
+		t.Fatal(err)
+	}
+	f.Decls = append(f.Decls, astFnClassfname(&gmxClass{clsfile: "get"}))
+	if f.End() != f.ShadowEntry.End() {
+		t.Fatal("class file end not shadow entry")
+	}
+}
+
 // -----------------------------------------------------------------------------
