@@ -41,11 +41,16 @@ type gmxClass struct {
 	proj    *gmxProject
 }
 
+type spxObj struct {
+	obj  gogen.Ref
+	proj string
+}
+
 type gmxProject struct {
-	gameClass  string               // <gmtype>.gmx
-	game       gogen.Ref            // Game (project base class)
-	sprite     map[string]gogen.Ref // .spx => Sprite
-	sptypes    []string             // <sptype>.spx
+	gameClass  string            // <gmtype>.gmx
+	game       gogen.Ref         // Game (project base class)
+	sprite     map[string]spxObj // .spx => Sprite
+	sptypes    []string          // <sptype>.spx
 	scheds     []string
 	schedStmts []goast.Stmt // nil or len(scheds) == 2 (delayload)
 	pkgImps    []gogen.PkgRef
@@ -54,6 +59,15 @@ type gmxProject struct {
 	hasScheds  bool
 	gameIsPtr  bool
 	isTest     bool
+	hasMain_   bool
+}
+
+func (p *gmxProject) hasMain() bool {
+	if !p.hasMain_ {
+		imps := p.pkgImps
+		p.hasMain_ = len(imps) > 0 && imps[0].TryRef("GopTestClass") != nil
+	}
+	return p.hasMain_
 }
 
 func (p *gmxProject) getScheds(cb *gogen.CodeBuilder) []goast.Stmt {
@@ -87,6 +101,32 @@ func ClassNameAndExt(file string) (name, clsfile, ext string) {
 	return
 }
 
+// GetFileClassType get ast.File classType
+func GetFileClassType(file *ast.File, filename string, lookupClass func(ext string) (c *Project, ok bool)) (classType string, isTest bool) {
+	if file.IsClass {
+		var ext string
+		classType, _, ext = ClassNameAndExt(filename)
+		if file.IsNormalGox {
+			isTest = strings.HasSuffix(ext, "_test.gox")
+			if !isTest && classType == "main" {
+				classType = "_main"
+			}
+		} else {
+			isTest = strings.HasSuffix(ext, "test.gox")
+		}
+		if file.IsProj && classType == "main" {
+			if gt, ok := lookupClass(ext); ok {
+				classType = gt.Class
+			}
+		} else if isTest {
+			classType = casePrefix + testNameSuffix(classType)
+		}
+	} else if strings.HasSuffix(filename, "_test.gop") {
+		isTest = true
+	}
+	return
+}
+
 func isGoxTestFile(ext string) bool {
 	return strings.HasSuffix(ext, "test.gox")
 }
@@ -126,10 +166,10 @@ func loadClass(ctx *pkgCtx, pkg *gogen.Package, file string, f *ast.File, conf *
 		if gt.Class != "" {
 			p.game, p.gameIsPtr = spxRef(spx, gt.Class)
 		}
-		p.sprite = make(map[string]types.Object)
+		p.sprite = make(map[string]spxObj)
 		for _, v := range gt.Works {
 			obj, _ := spxRef(spx, v.Class)
-			p.sprite[v.Ext] = obj
+			p.sprite[v.Ext] = spxObj{obj, v.Project}
 		}
 		if x := getStringConst(spx, "Gop_sched"); x != "" {
 			p.scheds, p.hasScheds = strings.SplitN(x, ",", 2), true
@@ -143,6 +183,7 @@ func loadClass(ctx *pkgCtx, pkg *gogen.Package, file string, f *ast.File, conf *
 			log.Panicln("multiple project files found:", tname, p.gameClass)
 		}
 		p.gameClass = tname
+		p.hasMain_ = f.HasShadowEntry()
 	} else {
 		p.sptypes = append(p.sptypes, tname)
 	}
@@ -173,7 +214,7 @@ func spxTryRef(spx gogen.PkgRef, typ string) (obj types.Object, isPtr bool) {
 func spxRef(spx gogen.PkgRef, typ string) (obj gogen.Ref, isPtr bool) {
 	obj, isPtr = spxTryRef(spx, typ)
 	if obj == nil {
-		panic(spx.Path() + "." + typ + " not found")
+		panic(spx.Types.Name() + "." + typ + " not found")
 	}
 	return
 }
@@ -246,20 +287,34 @@ func genTestFunc(pkg *gogen.Package, name, testType, param, paramType string) {
 		End()
 }
 
-func gmxCheckProjs(pkg *gogen.Package, ctx *pkgCtx) (proj *gmxProject, multi bool) {
+func gmxCheckProjs(pkg *gogen.Package, ctx *pkgCtx) (*gmxProject, bool) {
+	var projMain, projNoMain *gmxProject
+	var multiMain, multiNoMain bool
 	for _, v := range ctx.projs {
 		if v.isTest {
 			continue
-		} else if proj != nil {
-			multi = true
+		}
+		if v.hasMain() {
+			if projMain != nil {
+				multiMain = true
+			} else {
+				projMain = v
+			}
 		} else {
-			proj = v
+			if projNoMain != nil {
+				multiNoMain = true
+			} else {
+				projNoMain = v
+			}
 		}
 		if v.game != nil { // just to make testcase happy
 			gmxProjMain(pkg, ctx, v)
 		}
 	}
-	return
+	if projMain != nil {
+		return projMain, multiMain
+	}
+	return projNoMain, multiNoMain
 }
 
 func gmxProjMain(pkg *gogen.Package, parent *pkgCtx, proj *gmxProject) {
@@ -339,14 +394,12 @@ func gmxProjMain(pkg *gogen.Package, parent *pkgCtx, proj *gmxProject) {
 	})
 }
 
-func gmxMainFunc(pkg *gogen.Package, proj *gmxProject) func() {
-	return func() {
-		if o := pkg.TryRef(proj.gameClass); o != nil {
-			// new(gameClass).Main()
-			new := pkg.Builtin().Ref("new")
-			pkg.NewFunc(nil, "main", nil, nil, false).
-				BodyStart(pkg).Val(new).Val(o).Call(1).MemberVal("Main").Call(0).EndStmt().End()
-		}
+func genMainFunc(pkg *gogen.Package, gameClass string) {
+	if o := pkg.TryRef(gameClass); o != nil {
+		// new(gameClass).Main()
+		new := pkg.Builtin().Ref("new")
+		pkg.NewFunc(nil, "main", nil, nil, false).
+			BodyStart(pkg).Val(new).Val(o).Call(1).MemberVal("Main").Call(0).EndStmt().End()
 	}
 }
 
@@ -403,6 +456,7 @@ func astFnClassfname(c *gmxClass) *ast.FuncDecl {
 				},
 			},
 		},
+		Shadow: true,
 	}
 }
 
