@@ -17,125 +17,30 @@
 package cl_test
 
 import (
-	"bytes"
-	"os"
 	"testing"
 
 	"github.com/goplus/gop/cl"
-	"github.com/goplus/gop/parser"
-	"github.com/goplus/gop/parser/fsx/memfs"
-	"github.com/goplus/gop/scanner"
-	"github.com/goplus/mod/modfile"
+	"github.com/goplus/gop/cl/cltest"
 )
 
-func lookupClass(ext string) (c *modfile.Project, ok bool) {
-	switch ext {
-	case ".tgmx", ".tspx":
-		return &modfile.Project{
-			Ext: ".tgmx", Class: "*MyGame",
-			Works:    []*modfile.Class{{Ext: ".tspx", Class: "Sprite"}},
-			PkgPaths: []string{"github.com/goplus/gop/cl/internal/spx", "math"}}, true
-	case ".t2gmx", ".t2spx", ".t2spx2":
-		return &modfile.Project{
-			Ext: ".t2gmx", Class: "Game",
-			Works: []*modfile.Class{{Ext: ".t2spx", Class: "Sprite"},
-				{Ext: ".t2spx2", Class: "Sprite2"}},
-			PkgPaths: []string{"github.com/goplus/gop/cl/internal/spx2"}}, true
-	case "_t3spx.gox", ".t3spx2":
-		return &modfile.Project{
-			Works: []*modfile.Class{{Ext: "_t3spx.gox", Class: "Sprite"},
-				{Ext: ".t3spx2", Class: "Sprite2"}},
-			PkgPaths: []string{"github.com/goplus/gop/cl/internal/spx2"}}, true
-	case "_spx.gox":
-		return &modfile.Project{
-			Ext: "_spx.gox", Class: "Game",
-			Works:    []*modfile.Class{{Ext: "_spx.gox", Class: "Sprite"}},
-			PkgPaths: []string{"github.com/goplus/gop/cl/internal/spx3", "math"},
-			Import:   []*modfile.Import{{Path: "github.com/goplus/gop/cl/internal/spx3/jwt"}}}, true
-	case "_xtest.gox":
-		return &modfile.Project{
-			Ext: "_xtest.gox", Class: "App",
-			Works:    []*modfile.Class{{Ext: "_xtest.gox", Class: "Case"}},
-			PkgPaths: []string{"github.com/goplus/gop/test", "testing"}}, true
-	}
-	return
-}
-
-func spxParserConf() parser.Config {
-	return parser.Config{
-		ClassKind: func(fname string) (isProj bool, ok bool) {
-			ext := modfile.ClassExt(fname)
-			c, ok := lookupClass(ext)
-			if ok {
-				isProj = c.IsProj(ext, fname)
-			}
-			return
-		},
-	}
-}
-
 func gopSpxTest(t *testing.T, gmx, spxcode, expected string) {
-	gopSpxTestEx(t, gmx, spxcode, expected, "index.tgmx", "bar.tspx")
+	cltest.SpxEx(t, gmx, spxcode, expected, "index.tgmx", "bar.tspx")
 }
 
 func gopSpxTestEx(t *testing.T, gmx, spxcode, expected, gmxfile, spxfile string) {
-	gopSpxTestExConf(t, "gopSpxTest", gblConf, gmx, spxcode, expected, gmxfile, spxfile, "")
+	cltest.SpxWithConf(t, "gopSpxTest", cltest.Conf, gmx, spxcode, expected, gmxfile, spxfile, "")
 }
 
 func gopSpxTestEx2(t *testing.T, gmx, spxcode, expected, gmxfile, spxfile, resultFile string) {
-	gopSpxTestExConf(t, "gopSpxTest", gblConf, gmx, spxcode, expected, gmxfile, spxfile, resultFile)
+	cltest.SpxWithConf(t, "gopSpxTest", cltest.Conf, gmx, spxcode, expected, gmxfile, spxfile, resultFile)
 }
 
 func gopSpxTestExConf(t *testing.T, name string, conf *cl.Config, gmx, spxcode, expected, gmxfile, spxfile, resultFile string) {
-	t.Run(name, func(t *testing.T) {
-		cl.SetDisableRecover(true)
-		defer cl.SetDisableRecover(false)
-
-		fs := memfs.TwoFiles("/foo", spxfile, spxcode, gmxfile, gmx)
-		if gmxfile == "" {
-			fs = memfs.SingleFile("/foo", spxfile, spxcode)
-		}
-		pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", spxParserConf())
-		if err != nil {
-			scanner.PrintError(os.Stderr, err)
-			t.Fatal("ParseFSDir:", err)
-		}
-		bar := pkgs["main"]
-		pkg, err := cl.NewPackage("", bar, conf)
-		if err != nil {
-			t.Fatal("NewPackage:", err)
-		}
-		var b bytes.Buffer
-		err = pkg.WriteTo(&b, resultFile)
-		if err != nil {
-			t.Fatal("gogen.WriteTo failed:", err)
-		}
-		result := b.String()
-		if result != expected {
-			t.Fatalf("\nResult:\n%s\nExpected:\n%s\n", result, expected)
-		}
-	})
+	cltest.SpxWithConf(t, name, conf, gmx, spxcode, expected, gmxfile, spxfile, resultFile)
 }
 
 func gopSpxErrorTestEx(t *testing.T, msg, gmx, spxcode, gmxfile, spxfile string) {
-	fs := memfs.TwoFiles("/foo", spxfile, spxcode, gmxfile, gmx)
-	pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", spxParserConf())
-	if err != nil {
-		scanner.PrintError(os.Stderr, err)
-		t.Fatal("ParseFSDir:", err)
-	}
-	conf := *gblConf
-	conf.RelativeBase = "/foo"
-	conf.Recorder = nil
-	conf.NoFileLine = false
-	bar := pkgs["main"]
-	_, err = cl.NewPackage("", bar, &conf)
-	if err == nil {
-		t.Fatal("no error?")
-	}
-	if ret := err.Error(); ret != msg {
-		t.Fatalf("\nError: \"%s\"\nExpected: \"%s\"\n", ret, msg)
-	}
+	cltest.SpxErrorEx(t, msg, gmx, spxcode, gmxfile, spxfile)
 }
 
 func TestSpxNoGame(t *testing.T) {
@@ -793,7 +698,7 @@ func (this *Kai) Main() {
 }
 
 func TestSpxMainEntry(t *testing.T) {
-	conf := *gblConf
+	conf := *cltest.Conf
 	conf.Importer = nil
 	conf.NoAutoGenMain = false
 
@@ -1387,6 +1292,42 @@ type foo struct {
 `, "foo.gox")
 }
 
+func TestGopxOverload(t *testing.T) {
+	gopClTestFile(t, `
+func addString(a, b string) string {
+	return a + b
+}
+
+func addInt(a, b int) int {
+	return a + b
+}
+
+func add = (
+	addInt
+	func(a, b float64) float64 {
+		return a + b
+	}
+	addString
+)
+`, `package main
+
+const Gopo_Rect_add = ".addInt,,.addString"
+
+type Rect struct {
+}
+
+func (this *Rect) addString(a string, b string) string {
+	return a + b
+}
+func (this *Rect) addInt(a int, b int) int {
+	return a + b
+}
+func (this *Rect) add__1(a float64, b float64) float64 {
+	return a + b
+}
+`, "Rect.gox")
+}
+
 func TestClassFileGopx(t *testing.T) {
 	gopClTestFile(t, `
 var (
