@@ -17,20 +17,13 @@
 package cl_test
 
 import (
-	"bytes"
 	"os"
-	"strings"
 	"sync"
 	"testing"
 
 	"github.com/goplus/gogen"
-	"github.com/goplus/gop"
 	"github.com/goplus/gop/cl"
-	"github.com/goplus/gop/parser"
-	"github.com/goplus/gop/parser/fsx/memfs"
-	"github.com/goplus/gop/scanner"
-	"github.com/goplus/gop/token"
-	"github.com/goplus/mod/env"
+	"github.com/goplus/gop/cl/cltest"
 )
 
 const (
@@ -38,88 +31,40 @@ const (
 )
 
 var (
-	gblFset     *token.FileSet
-	gblConf     *cl.Config
 	gblConfLine *cl.Config
 )
 
 func init() {
-	gogen.SetDebug(gogen.DbgFlagAll)
-	cl.SetDebug(cl.DbgFlagAll | cl.FlagNoMarkAutogen)
-	gblFset = token.NewFileSet()
-	imp := gop.NewImporter(nil, &env.Gop{Root: gopRootDir, Version: "1.0"}, gblFset)
-	gblConf = &cl.Config{
-		Fset:          gblFset,
-		Importer:      imp,
-		Recorder:      gopRecorder{},
-		LookupClass:   lookupClass,
-		LookupPub:     lookupPub,
-		C2goBase:      "github.com/goplus/gop/cl/internal",
-		NoFileLine:    true,
-		NoAutoGenMain: true,
-	}
+	cltest.Gop.Root = gopRootDir
+	conf := cltest.Conf
 	gblConfLine = &cl.Config{
-		Fset:          gblFset,
-		Importer:      imp,
-		Recorder:      gopRecorder{},
-		LookupClass:   lookupClass,
-		LookupPub:     lookupPub,
-		C2goBase:      "github.com/goplus/gop/cl/internal",
+		Fset:          conf.Fset,
+		Importer:      conf.Importer,
+		Recorder:      cltest.Conf.Recorder,
+		LookupClass:   cltest.LookupClass,
 		NoFileLine:    false,
 		NoAutoGenMain: true,
 	}
 }
 
 func gopClNamedTest(t *testing.T, name string, gopcode, expected string) {
-	t.Run(name, func(t *testing.T) {
-		gopClTest(t, gopcode, expected)
-	})
+	cltest.Named(t, name, gopcode, expected)
 }
 
 func gopClTest(t *testing.T, gopcode, expected string) {
-	gopClTestEx(t, gblConf, "main", gopcode, expected)
+	cltest.DoExt(t, cltest.Conf, "main", gopcode, expected)
 }
 
 func gopClTestFile(t *testing.T, gopcode, expected string, fname string) {
-	fs := memfs.SingleFile("/foo", fname, gopcode)
-	gopClTestFS(t, gblConf, fs, "main", expected)
+	cltest.DoWithFname(t, gopcode, expected, fname)
 }
 
 func gopClTestEx(t *testing.T, conf *cl.Config, pkgname, gopcode, expected string) {
-	fs := memfs.SingleFile("/foo", "bar.gop", gopcode)
-	gopClTestFS(t, conf, fs, pkgname, expected)
+	cltest.DoExt(t, conf, pkgname, gopcode, expected)
 }
 
 func gopMixedClTest(t *testing.T, pkgname, gocode, gopcode, expected string, outline ...bool) {
-	conf := *gblConf
-	conf.Outline = (outline != nil && outline[0])
-	fs := memfs.TwoFiles("/foo", "a.go", gocode, "b.gop", gopcode)
-	gopClTestFS(t, &conf, fs, pkgname, expected)
-}
-
-func gopClTestFS(t *testing.T, conf *cl.Config, fs parser.FileSystem, pkgname, expected string) {
-	cl.SetDisableRecover(true)
-	defer cl.SetDisableRecover(false)
-
-	pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", parser.Config{Mode: parser.ParseComments})
-	if err != nil {
-		scanner.PrintError(os.Stderr, err)
-		t.Fatal("ParseFSDir:", err)
-	}
-	bar := pkgs[pkgname]
-	pkg, err := cl.NewPackage("github.com/goplus/gop/cl", bar, conf)
-	if err != nil {
-		t.Fatal("NewPackage:", err)
-	}
-	var b bytes.Buffer
-	err = pkg.WriteTo(&b)
-	if err != nil {
-		t.Fatal("gogen.WriteTo failed:", err)
-	}
-	result := b.String()
-	if result != expected {
-		t.Fatalf("\nResult:\n%s\nExpected:\n%s\n", result, expected)
-	}
+	cltest.Mixed(t, pkgname, gocode, gopcode, expected, outline...)
 }
 
 func TestTypeDoc(t *testing.T) {
@@ -1279,8 +1224,8 @@ func TestAutoProperty(t *testing.T) {
 func foo(script string) {
 	doc := goptest.New(script)!
 
-	println(doc.any.funcDecl.name)
-	println(doc.any.importSpec.name)
+	echo doc.any.funcDecl.name
+	echo doc.any.importSpec.name
 }
 `, `package main
 
@@ -1301,7 +1246,7 @@ func foo(script string) {
 		}
 		return
 	}()
-	fmt.Println(doc.Any().FuncDecl().Name())
+	fmt.Println(doc.Any().FuncDecl__0().Name())
 	fmt.Println(doc.Any().ImportSpec().Name())
 }
 `)
@@ -1335,7 +1280,7 @@ func foo(script string) {
 		}
 		return
 	}()
-	fmt.Println(doc.Any().FuncDecl().Name())
+	fmt.Println(doc.Any().FuncDecl__0().Name())
 	fmt.Println(doc.Any().ImportSpec().Name())
 }
 `)
@@ -3634,247 +3579,6 @@ func newRepo() Repo {
 	}
 }
 
-func testRangeExpr(t *testing.T, codeTpl, expect string) {
-	for k, s := range []string{" <- ", " := range ", " = range "} {
-		if k == 2 {
-			codeTpl = "i:=0\n" + codeTpl
-			expect = strings.Replace(expect, "for i := ", "i := 0\n\tfor i = ", -1)
-		}
-		gopClTest(t, strings.Replace(codeTpl, "$", s, -1), expect)
-	}
-}
-
-func TestRangeExpr(t *testing.T) {
-	testRangeExpr(t, `
-for i $ :10 {
-	println(i)
-}`, `package main
-
-import "fmt"
-
-func main() {
-	for i := 0; i < 10; i += 1 {
-		fmt.Println(i)
-	}
-}
-`)
-	testRangeExpr(t, `
-for i $ 1:10:3 {
-	println(i)
-}`, `package main
-
-import "fmt"
-
-func main() {
-	for i := 1; i < 10; i += 3 {
-		fmt.Println(i)
-	}
-}
-`)
-}
-
-func TestRangeExpr2(t *testing.T) {
-	testRangeExpr(t, `
-for i $ 1:10:2 {
-	println(i)
-}`, `package main
-
-import "fmt"
-
-func main() {
-	for i := 1; i < 10; i += 2 {
-		fmt.Println(i)
-	}
-}
-`)
-}
-
-func TestRangeExpr3(t *testing.T) {
-	testRangeExpr(t, `
-for i $ 1:10 {
-	println(i)
-}`, `package main
-
-import "fmt"
-
-func main() {
-	for i := 1; i < 10; i += 1 {
-		fmt.Println(i)
-	}
-}
-`)
-}
-
-func TestRangeExpr4(t *testing.T) {
-	testRangeExpr(t, `
-for i $ :10:2 {
-	println(i)
-}`, `package main
-
-import "fmt"
-
-func main() {
-	for i := 0; i < 10; i += 2 {
-		fmt.Println(i)
-	}
-}
-`)
-}
-
-func TestRangeExpr5(t *testing.T) {
-	gopClTest(t, `
-for range :10 {
-	println("Hi")
-}`, `package main
-
-import "fmt"
-
-func main() {
-	for _gop_k := 0; _gop_k < 10; _gop_k += 1 {
-		fmt.Println("Hi")
-	}
-}
-`)
-}
-
-func TestRangeExpr6(t *testing.T) {
-	gopClTest(t, `
-for _ <- :10 {
-	println("Hi")
-}`, `package main
-
-import "fmt"
-
-func main() {
-	for _gop_k := 0; _gop_k < 10; _gop_k += 1 {
-		fmt.Println("Hi")
-	}
-}
-`)
-}
-
-func TestRangeExpr7(t *testing.T) {
-	gopClTest(t, `
-println [x for x <- 0:3:1]
-`, `package main
-
-import (
-	"fmt"
-	"github.com/goplus/gop/builtin"
-)
-
-func main() {
-	fmt.Println(func() (_gop_ret []int) {
-		for _gop_it := builtin.NewRange__0(0, 3, 1).Gop_Enum(); ; {
-			var _gop_ok bool
-			x, _gop_ok := _gop_it.Next()
-			if !_gop_ok {
-				break
-			}
-			_gop_ret = append(_gop_ret, x)
-		}
-		return
-	}())
-}
-`)
-}
-
-func testRangeExpr8(t *testing.T, codeTpl, expect string) {
-	for _, s := range []string{" <- ", " := range "} {
-		gopClTest(t, strings.Replace(codeTpl, "$", s, -1), expect)
-	}
-}
-
-func TestRangeExpr8(t *testing.T) {
-	testRangeExpr8(t, `
-	type T struct{}
-
-	func (t T)start()int{
-		return 0
-	}
-	func (t T)end()int{
-		return 3 
-	}
-	func (t T)step()int{
-		return 1 
-	}
-	
-	t:=T{}
-	
-	for i <- t.start():t.end():t.step(){
-		println i
-	}
-	`, `package main
-
-import "fmt"
-
-type T struct {
-}
-
-func (t T) start() int {
-	return 0
-}
-func (t T) end() int {
-	return 3
-}
-func (t T) step() int {
-	return 1
-}
-func main() {
-	t := T{}
-	for i, _gop_end, _gop_step := t.start(), t.end(), t.step(); i < _gop_end; i += _gop_step {
-		fmt.Println(i)
-	}
-}
-`)
-}
-
-func TestRangeExpr9(t *testing.T) {
-	testRangeExpr8(t, `
-	type T struct{}
-
-	func (t T)start()int{
-		return 0
-	}
-	func (t T)end()int{
-		return 3 
-	}
-	func (t T)step()int{
-		return 1 
-	}
-	
-	t:=T{}
-	i:=0
-	for i =range t.start():t.end():t.step(){
-		println i
-	}
-	`, `package main
-
-import "fmt"
-
-type T struct {
-}
-
-func (t T) start() int {
-	return 0
-}
-func (t T) end() int {
-	return 3
-}
-func (t T) step() int {
-	return 1
-}
-func main() {
-	t := T{}
-	i := 0
-	for _gop_k, _gop_end, _gop_step := t.start(), t.end(), t.step(); _gop_k < _gop_end; _gop_k += _gop_step {
-		i = _gop_k
-		fmt.Println(i)
-	}
-}
-`)
-}
-
 func TestGoFuncInstr(t *testing.T) {
 	gopClTest(t, `package main
 
@@ -3912,7 +3616,7 @@ func main() {
 	fmt.Println("init")
 }
 `)
-	gopClTestEx(t, gblConf, "bar", `package bar
+	gopClTestEx(t, cltest.Conf, "bar", `package bar
 println("init")
 `, `package bar
 
@@ -4015,7 +3719,7 @@ func main() {
 }
 
 func TestMainEntry(t *testing.T) {
-	conf := *gblConf
+	conf := *cltest.Conf
 	conf.NoAutoGenMain = false
 
 	gopClTestEx(t, &conf, "main", `
@@ -4141,7 +3845,7 @@ func main() {
 }
 
 func TestCommentLineRoot(t *testing.T) {
-	conf := *gblConf
+	conf := *cltest.Conf
 	conf.NoFileLine = false
 	conf.RelativeBase = "/foo/root"
 	var src = `
@@ -4418,3 +4122,132 @@ func _() {
 }
 `)
 }
+
+func TestSliceLitAssign(t *testing.T) {
+	gopClTest(t, `
+var n = 1
+var a []any = [10, 3.14, 200]
+n, a = 100, [10, 3.14, 200]
+echo a, n
+`, `package main
+
+import "fmt"
+
+var n = 1
+var a []interface{} = []interface{}{10, 3.14, 200}
+
+func main() {
+	n, a = 100, []interface{}{10, 3.14, 200}
+	fmt.Println(a, n)
+}
+`)
+}
+
+func TestSliceLitReturn(t *testing.T) {
+	gopClTest(t, `
+func anyslice() (int, []any) {
+	return 100, [10, 3.14, 200]
+}
+n, a := anyslice()
+echo n, a
+`, `package main
+
+import "fmt"
+
+func anyslice() (int, []interface{}) {
+	return 100, []interface{}{10, 3.14, 200}
+}
+func main() {
+	n, a := anyslice()
+	fmt.Println(n, a)
+}
+`)
+}
+
+func TestCompositeLitAssign(t *testing.T) {
+	gopClTest(t, `
+var a map[any]any = {10: "A", 3.14: "B", 200: "C"}
+var b map[any]string = {10: "A", 3.14: "B", 200: "C"}
+echo a
+echo b
+var n int
+n, a = 1, {10: "A", 3.14: "B", 200: "C"}
+echo a, n
+n, b = 1, {10: "A", 3.14: "B", 200: "C"}
+echo b, n
+`, `package main
+
+import "fmt"
+
+var a map[interface{}]interface{} = map[interface{}]interface{}{10: "A", 3.14: "B", 200: "C"}
+var b map[interface{}]string = map[interface{}]string{10: "A", 3.14: "B", 200: "C"}
+
+func main() {
+	fmt.Println(a)
+	fmt.Println(b)
+	var n int
+	n, a = 1, map[interface{}]interface{}{10: "A", 3.14: "B", 200: "C"}
+	fmt.Println(a, n)
+	n, b = 1, map[interface{}]string{10: "A", 3.14: "B", 200: "C"}
+	fmt.Println(b, n)
+}
+`)
+}
+
+func TestCompositeLitStruct(t *testing.T) {
+	gopClTest(t, `
+type T struct {
+	s  []any
+	m  map[any]any
+	fn func(int) int
+}
+
+echo &T{[10, 3.14, 200], {10: "A", 3.14: "B", 200: "C"}, (x => x)}
+echo &T{s: [10, 3.14, 200], m: {10: "A", 3.14: "B", 200: "C"}, fn: (x => x)}
+`, `package main
+
+import "fmt"
+
+type T struct {
+	s  []interface{}
+	m  map[interface{}]interface{}
+	fn func(int) int
+}
+
+func main() {
+	fmt.Println(&T{[]interface{}{10, 3.14, 200}, map[interface{}]interface{}{10: "A", 3.14: "B", 200: "C"}, func(x int) int {
+		return x
+	}})
+	fmt.Println(&T{s: []interface{}{10, 3.14, 200}, m: map[interface{}]interface{}{10: "A", 3.14: "B", 200: "C"}, fn: func(x int) int {
+		return x
+	}})
+}
+`)
+}
+
+func TestCompositeLitEx(t *testing.T) {
+	gopClTest(t, `
+var a [][]any = {[10, 3.14, 200], [100, 200]}
+var m map[any][]any = {10: [10, 3.14, 200]}
+var f map[any]func(int) int = {10: x => x}
+
+echo a
+echo m
+echo f
+`, `package main
+
+import "fmt"
+
+var a [][]interface{} = [][]interface{}{[]interface{}{10, 3.14, 200}, []interface{}{100, 200}}
+var m map[interface{}][]interface{} = map[interface{}][]interface{}{10: []interface{}{10, 3.14, 200}}
+var f map[interface{}]func(int) int = map[interface{}]func(int) int{10: func(x int) int {
+	return x
+}}
+
+func main() {
+	fmt.Println(a)
+	fmt.Println(m)
+	fmt.Println(f)
+}
+`)
+}
