@@ -18,18 +18,17 @@ package cl
 
 import (
 	"bytes"
+	"errors"
 	goast "go/ast"
 	gotoken "go/token"
 	"go/types"
 	"log"
 	"math/big"
-	"reflect"
 	"strconv"
 	"strings"
 	"syscall"
 
 	"github.com/goplus/gogen"
-	"github.com/goplus/gogen/cpackages"
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/printer"
 	"github.com/goplus/gop/token"
@@ -72,7 +71,6 @@ const (
 const (
 	objNormal = iota
 	objPkgRef
-	objCPkgRef
 	objGopExecOrEnv
 
 	objGopEnv  = objGopExecOrEnv
@@ -125,10 +123,6 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) (pkg gogen.PkgRef,
 
 	// pkgRef object
 	if (flags & clIdentSelectorExpr) != 0 {
-		if name == "C" && len(ctx.clookups) > 0 {
-			kind = objCPkgRef
-			return
-		}
 		if pi, ok := ctx.findImport(name); ok {
 			if rec := ctx.recorder(); rec != nil {
 				rec.Use(ident, pi.pkgName)
@@ -188,6 +182,34 @@ find:
 	return
 }
 
+/*
+func compileMatrixLit(ctx *blockCtx, v *ast.MatrixLit) {
+	cb := ctx.cb
+	ncol := -1
+	for _, elts := range v.Elts {
+		switch n := len(elts); n {
+		case 1:
+			elt := elts[0]
+			if e, ok := elt.(*ast.Ellipsis); ok {
+				compileExpr(ctx, e.Elt)
+				panic("TODO") // TODO(xsw): matrixLit with ellipsis
+			}
+			fallthrough
+		default:
+			if ncol < 0 {
+				ncol = n
+			} else if ncol != n {
+				ctx.handleErrorf(elts[0].Pos(), "inconsistent matrix column count: got %v, want %v", n, ncol)
+			}
+			for _, elt := range elts {
+				compileExpr(ctx, elt)
+			}
+			cb.SliceLitEx(...)
+		}
+	}
+}
+*/
+
 func compileEnvExpr(ctx *blockCtx, v *ast.EnvExpr) {
 	cb := ctx.cb
 	if ctx.isClass { // in a Go+ class file
@@ -248,7 +270,7 @@ func compileExprLHS(ctx *blockCtx, expr ast.Expr) {
 	case *ast.StarExpr:
 		compileStarExprLHS(ctx, v)
 	default:
-		log.Panicln("compileExpr failed: unknown -", reflect.TypeOf(v))
+		panic(ctx.newCodeErrorf(v.Pos(), "compileExprLHS failed: unknown - %T", expr))
 	}
 	if rec := ctx.recorder(); rec != nil {
 		rec.recordExpr(ctx, expr, true)
@@ -363,8 +385,12 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, inFlags ...int) {
 		ctx.cb.Typ(toFuncType(ctx, v, nil, nil), v)
 	case *ast.EnvExpr:
 		compileEnvExpr(ctx, v)
+	/* case *ast.MatrixLit:
+	compileMatrixLit(ctx, v) */
+	case *ast.DomainTextLit:
+		compileDomainTextLit(ctx, v)
 	default:
-		log.Panicf("compileExpr failed: unknown - %T\n", v)
+		panic(ctx.newCodeErrorf(v.Pos(), "compileExpr failed: unknown - %T", v))
 	}
 	if rec := ctx.recorder(); rec != nil {
 		rec.recordExpr(ctx, expr, false)
@@ -515,18 +541,6 @@ func lookupPkgRef(ctx *blockCtx, pkg gogen.PkgRef, x *ast.Ident, pkgKind int) (o
 				pkg, o, alias = at, o2, alias2
 			}
 		}
-	} else {
-		var cpkg cpackages.PkgRef
-		for _, at := range ctx.clookups {
-			if o2 := at.Lookup(x.Name); o2 != nil {
-				if o != nil {
-					panic(ctx.newCodeErrorf(
-						x.Pos(), "confliction: %s declared both in \"%s\" and \"%s\"",
-						x.Name, at.Pkg().Types.Path(), cpkg.Pkg().Types.Path()))
-				}
-				cpkg, o = at, o2
-			}
-		}
 	}
 	return
 }
@@ -566,12 +580,14 @@ func identVal(ctx *blockCtx, x *ast.Ident, flags int, v types.Object, alias bool
 }
 
 type fnType struct {
-	next     *fnType
-	params   *types.Tuple
-	base     int
-	size     int
-	variadic bool
-	typetype bool
+	next      *fnType
+	params    *types.Tuple
+	sig       *types.Signature
+	base      int
+	size      int
+	variadic  bool
+	typetype  bool
+	typeparam bool
 }
 
 func (p *fnType) arg(i int, ellipsis bool) types.Type {
@@ -590,7 +606,8 @@ func (p *fnType) arg(i int, ellipsis bool) types.Type {
 
 func (p *fnType) init(base int, t *types.Signature) {
 	p.base = base
-	p.params, p.variadic = t.Params(), t.Variadic()
+	p.sig = t
+	p.params, p.variadic, p.typeparam = t.Params(), t.Variadic(), t.TypeParams() != nil
 	p.size = p.params.Len()
 	if p.variadic {
 		p.size--
@@ -669,7 +686,6 @@ func compileCallExpr(ctx *blockCtx, v *ast.CallExpr, inFlags int) {
 	var err error
 	var stk = ctx.cb.InternalStack()
 	var base = stk.Len()
-	var fnt = stk.Get(-1).Type
 	var flags gogen.InstrFlags
 	var ellipsis = v.Ellipsis != gotoken.NoPos
 	if ellipsis {
@@ -678,10 +694,12 @@ func compileCallExpr(ctx *blockCtx, v *ast.CallExpr, inFlags int) {
 	if (inFlags & clCallWithTwoValue) != 0 {
 		flags |= gogen.InstrFlagTwoValue
 	}
+	pfn := stk.Get(-1)
+	fnt := pfn.Type
 	fn := &fnType{}
 	fn.load(fnt)
 	for fn != nil {
-		if err = compileCallArgs(fn, ctx, v, ellipsis, flags); err == nil {
+		if err = compileCallArgs(ctx, pfn, fn, v, ellipsis, flags); err == nil {
 			if rec := ctx.recorder(); rec != nil {
 				rec.recordCallExpr(ctx, v, fnt)
 			}
@@ -736,10 +754,22 @@ func fnCall(ctx *blockCtx, v *ast.CallExpr, flags gogen.InstrFlags, extra int) e
 	return ctx.cb.CallWithEx(len(v.Args)+extra, flags, v)
 }
 
-func compileCallArgs(fn *fnType, ctx *blockCtx, v *ast.CallExpr, ellipsis bool, flags gogen.InstrFlags) (err error) {
+func compileCallArgs(ctx *blockCtx, pfn *gogen.Element, fn *fnType, v *ast.CallExpr, ellipsis bool, flags gogen.InstrFlags) (err error) {
+	defer func() {
+		r := recover()
+		if r != nil {
+			err = ctx.recoverErr(r, v)
+		}
+	}()
+	var needInferFunc bool
 	for i, arg := range v.Args {
 		switch expr := arg.(type) {
 		case *ast.LambdaExpr:
+			if fn.typeparam {
+				needInferFunc = true
+				compileIdent(ctx, ast.NewIdent("nil"), 0)
+				continue
+			}
 			sig, e := checkLambdaFuncType(ctx, expr, fn.arg(i, ellipsis), clLambaArgument, v.Fun)
 			if e != nil {
 				return e
@@ -748,6 +778,11 @@ func compileCallArgs(fn *fnType, ctx *blockCtx, v *ast.CallExpr, ellipsis bool,
 				return
 			}
 		case *ast.LambdaExpr2:
+			if fn.typeparam {
+				needInferFunc = true
+				compileIdent(ctx, ast.NewIdent("nil"), 0)
+				continue
+			}
 			sig, e := checkLambdaFuncType(ctx, expr, fn.arg(i, ellipsis), clLambaArgument, v.Fun)
 			if e != nil {
 				return e
@@ -780,13 +815,31 @@ func compileCallArgs(fn *fnType, ctx *blockCtx, v *ast.CallExpr, ellipsis bool,
 			if typetype {
 				return
 			}
+		case *ast.NumberUnitLit:
+			compileNumberUnitLit(ctx, expr, fn.arg(i, ellipsis))
 		default:
 			compileExpr(ctx, arg)
 		}
 	}
+	if needInferFunc {
+		args := ctx.cb.InternalStack().GetArgs(len(v.Args))
+		typ, err := gogen.InferFunc(ctx.pkg, pfn, fn.sig, nil, args, flags)
+		if err != nil {
+			return err
+		}
+		next := &fnType{}
+		next.init(fn.base, typ.(*types.Signature))
+		next.next = fn.next
+		fn.next = next
+		return errCallNext
+	}
 	return ctx.cb.CallWithEx(len(v.Args), flags, v)
 }
 
+var (
+	errCallNext = errors.New("call next")
+)
+
 type clLambaFlag string
 
 const (
@@ -811,8 +864,11 @@ retry:
 		typ = t.Underlying()
 		goto retry
 	}
-	src := ctx.LoadExpr(toNode)
-	return nil, ctx.newCodeErrorf(lambda.Pos(), "cannot use lambda literal as type %v in %v to %v", ftyp, flag, src)
+	var to string
+	if toNode != nil {
+		to = " to " + ctx.LoadExpr(toNode)
+	}
+	return nil, ctx.newCodeErrorf(lambda.Pos(), "cannot use lambda literal as type %v in %v%v", ftyp, flag, to)
 }
 
 func compileLambda(ctx *blockCtx, lambda ast.Expr, sig *types.Signature) {
@@ -926,26 +982,32 @@ func compileFuncLit(ctx *blockCtx, v *ast.FuncLit) {
 	}
 }
 
+func compileNumberUnitLit(ctx *blockCtx, v *ast.NumberUnitLit, expected types.Type) {
+	ctx.cb.ValWithUnit(
+		&goast.BasicLit{ValuePos: v.ValuePos, Kind: gotoken.Token(v.Kind), Value: v.Value},
+		expected, v.Unit)
+}
+
 func compileBasicLit(ctx *blockCtx, v *ast.BasicLit) {
 	cb := ctx.cb
-	switch v.Kind {
+	switch kind := v.Kind; kind {
 	case token.RAT:
 		val := v.Value
 		bi, _ := new(big.Int).SetString(val[:len(val)-1], 10) // remove r suffix
 		cb.UntypedBigInt(bi, v)
-	case token.CSTRING:
+	case token.CSTRING, token.PYSTRING:
 		s, err := strconv.Unquote(v.Value)
 		if err != nil {
 			log.Panicln("compileBasicLit:", err)
 		}
-		n := len(s)
-		tyInt8 := types.Typ[types.Int8]
-		typ := types.NewArray(tyInt8, int64(n+1))
-		cb.Typ(types.NewPointer(tyInt8)).Typ(types.Typ[types.UnsafePointer])
-		for i := 0; i < n; i++ {
-			cb.Val(rune(s[i]))
+		var xstr gogen.Ref
+		switch kind {
+		case token.CSTRING:
+			xstr = ctx.cstr()
+		default:
+			xstr = ctx.pystr()
 		}
-		cb.Val(rune(0)).ArrayLit(typ, n+1).UnaryOp(gotoken.AND).Call(1).Call(1)
+		cb.Val(xstr).Val(s).Call(1)
 	default:
 		if v.Extra == nil {
 			basicLit(cb, v)
@@ -993,10 +1055,12 @@ func compileStringLitEx(ctx *blockCtx, cb *gogen.CodeBuilder, lit *ast.BasicLit)
 			t := cb.Get(-1).Type
 			if t.Underlying() != types.Typ[types.String] {
 				if _, err := cb.Member("string", gogen.MemberFlagAutoProperty); err != nil {
-					if e, ok := err.(*gogen.CodeError); ok {
-						err = ctx.newCodeErrorf(v.Pos(), "%s.string%s", ctx.LoadExpr(v), e.Msg)
+					if _, e2 := cb.Member("error", gogen.MemberFlagAutoProperty); e2 != nil {
+						if e, ok := err.(*gogen.CodeError); ok {
+							err = ctx.newCodeErrorf(v.Pos(), "%s.string%s", ctx.LoadExpr(v), e.Msg)
+						}
+						ctx.handleErr(err)
 					}
-					ctx.handleErr(err)
 				}
 			}
 			pos = v.End()
@@ -1009,6 +1073,19 @@ func compileStringLitEx(ctx *blockCtx, cb *gogen.CodeBuilder, lit *ast.BasicLit)
 	}
 }
 
+const (
+	tplPkgPath = "github.com/goplus/gop/tpl"
+)
+
+func compileDomainTextLit(ctx *blockCtx, v *ast.DomainTextLit) {
+	switch v.Domain.Name {
+	case "tpl": // tpl`...` => tpl.new(`...`)
+		ctx.cb.Val(ctx.pkg.Import(tplPkgPath).Ref("New")).
+			Val(&goast.BasicLit{Kind: gotoken.STRING, Value: v.Value}, v).
+			Call(1)
+	}
+}
+
 const (
 	compositeLitVal    = 0
 	compositeLitKeyVal = 1
@@ -1023,7 +1100,7 @@ func checkCompositeLitElts(elts []ast.Expr) (kind int) {
 	return compositeLitVal
 }
 
-func compileCompositeLitElts(ctx *blockCtx, elts []ast.Expr, kind int, expected *kvType) {
+func compileCompositeLitElts(ctx *blockCtx, elts []ast.Expr, kind int, expected *kvType) error {
 	for _, elt := range elts {
 		if kv, ok := elt.(*ast.KeyValueExpr); ok {
 			if key, ok := kv.Key.(*ast.CompositeLit); ok && key.Type == nil {
@@ -1031,22 +1108,60 @@ func compileCompositeLitElts(ctx *blockCtx, elts []ast.Expr, kind int, expected
 			} else {
 				compileExpr(ctx, kv.Key)
 			}
-			if val, ok := kv.Value.(*ast.CompositeLit); ok && val.Type == nil {
-				compileCompositeLit(ctx, val, expected.Elem(), false)
-			} else {
-				compileExpr(ctx, kv.Value)
+			err := compileCompositeLitElt(ctx, kv.Value, expected.Elem(), clLambaAssign, kv.Key)
+			if err != nil {
+				return err
 			}
 		} else {
 			if kind == compositeLitKeyVal {
 				ctx.cb.None()
 			}
-			if val, ok := elt.(*ast.CompositeLit); ok && val.Type == nil {
-				compileCompositeLit(ctx, val, expected.Elem(), false)
-			} else {
-				compileExpr(ctx, elt)
+			err := compileCompositeLitElt(ctx, elt, expected.Elem(), clLambaAssign, nil)
+			if err != nil {
+				return err
 			}
 		}
 	}
+	return nil
+}
+
+func compileCompositeLitElt(ctx *blockCtx, e ast.Expr, typ types.Type, flag clLambaFlag, toNode ast.Node) error {
+	switch v := unparen(e).(type) {
+	case *ast.LambdaExpr, *ast.LambdaExpr2:
+		sig, err := checkLambdaFuncType(ctx, v, typ, flag, toNode)
+		if err != nil {
+			return err
+		}
+		compileLambda(ctx, v, sig)
+	case *ast.SliceLit:
+		compileSliceLit(ctx, v, typ)
+	case *ast.CompositeLit:
+		compileCompositeLit(ctx, v, typ, false)
+	default:
+		compileExpr(ctx, v)
+	}
+	return nil
+}
+
+func unparen(x ast.Expr) ast.Expr {
+	if e, ok := x.(*ast.ParenExpr); ok {
+		return e.X
+	}
+	return x
+}
+
+func compileStructLit(ctx *blockCtx, elts []ast.Expr, t *types.Struct, typ types.Type, src *ast.CompositeLit) error {
+	for idx, elt := range elts {
+		if idx >= t.NumFields() {
+			return ctx.newCodeErrorf(elt.Pos(), "too many values in %v{...}", typ)
+		}
+		err := compileCompositeLitElt(ctx, elt, t.Field(idx).Type(), clLambaField, nil)
+		if err != nil {
+			return err
+		}
+	}
+	ctx.cb.StructLit(typ, len(elts), false, src)
+	return nil
 }
 
 func compileStructLitInKeyVal(ctx *blockCtx, elts []ast.Expr, t *types.Struct, typ types.Type, src *ast.CompositeLit) error {
@@ -1063,15 +1178,9 @@ func compileStructLitInKeyVal(ctx *blockCtx, elts []ast.Expr, t *types.Struct, t
 		if rec := ctx.recorder(); rec != nil {
 			rec.Use(name, t.Field(idx))
 		}
-		switch expr := kv.Value.(type) {
-		case *ast.LambdaExpr, *ast.LambdaExpr2:
-			sig, err := checkLambdaFuncType(ctx, expr, t.Field(idx).Type(), clLambaField, kv.Key)
-			if err != nil {
-				return err
-			}
-			compileLambda(ctx, expr, sig)
-		default:
-			compileExpr(ctx, kv.Value)
+		err := compileCompositeLitElt(ctx, kv.Value, t.Field(idx).Type(), clLambaField, kv.Key)
+		if err != nil {
+			return err
 		}
 	}
 	ctx.cb.StructLit(typ, len(elts)<<1, true, src)
@@ -1152,12 +1261,21 @@ func compileCompositeLitEx(ctx *blockCtx, v *ast.CompositeLit, expected types.Ty
 			typ, underlying = expected, tu
 		}
 	}
-	if t, ok := underlying.(*types.Struct); ok && kind == compositeLitKeyVal {
-		if err := compileStructLitInKeyVal(ctx, v.Elts, t, typ, v); err != nil {
+	if t, ok := underlying.(*types.Struct); ok {
+		var err error
+		if kind == compositeLitKeyVal {
+			err = compileStructLitInKeyVal(ctx, v.Elts, t, typ, v)
+		} else {
+			err = compileStructLit(ctx, v.Elts, t, typ, v)
+		}
+		if err != nil {
 			return err
 		}
 	} else {
-		compileCompositeLitElts(ctx, v.Elts, kind, &kvType{underlying: underlying})
+		err := compileCompositeLitElts(ctx, v.Elts, kind, &kvType{underlying: underlying})
+		if err != nil {
+			return err
+		}
 		n := len(v.Elts)
 		if isMap(underlying) {
 			if kind == compositeLitVal && n > 0 {
@@ -1172,8 +1290,6 @@ func compileCompositeLitEx(ctx *blockCtx, v *ast.CompositeLit, expected types.Ty
 				ctx.cb.SliceLitEx(typ, n<<kind, kind == compositeLitKeyVal, v)
 			case *types.Array:
 				ctx.cb.ArrayLitEx(typ, n<<kind, kind == compositeLitKeyVal, v)
-			case *types.Struct:
-				ctx.cb.StructLit(typ, n, false, v) // key-val mode handled by compileStructLitInKeyVal
 			default:
 				return ctx.newCodeErrorf(v.Pos(), "invalid composite literal type %v", typ)
 			}
