@@ -26,9 +26,9 @@ import (
 	gotoken "go/token"
 	"go/types"
 
+	"github.com/goplus/gogen"
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/token"
-	"github.com/goplus/gox"
 )
 
 func fileLineFile(relBaseDir, absFile string) string {
@@ -51,8 +51,15 @@ func commentStmt(ctx *blockCtx, stmt ast.Stmt) {
 	}
 }
 
-func commentStmtEx(cb *gox.CodeBuilder, ctx *pkgCtx, stmt ast.Stmt) {
+func commentStmtEx(cb *gogen.CodeBuilder, ctx *pkgCtx, stmt ast.Stmt) {
 	start := stmt.Pos()
+	if start == token.NoPos {
+		cb.SetComments(nil, false)
+		return
+	}
+	if doc := checkStmtDoc(stmt); doc != nil {
+		start = doc.Pos()
+	}
 	pos := ctx.fset.Position(start)
 	if ctx.relBaseDir != "" {
 		pos.Filename = fileLineFile(ctx.relBaseDir, pos.Filename)
@@ -64,7 +71,16 @@ func commentStmtEx(cb *gox.CodeBuilder, ctx *pkgCtx, stmt ast.Stmt) {
 	cb.SetComments(comments, false)
 }
 
-func commentFunc(ctx *blockCtx, fn *gox.Func, decl *ast.FuncDecl) {
+func checkStmtDoc(stmt ast.Stmt) *ast.CommentGroup {
+	if decl, ok := stmt.(*ast.DeclStmt); ok {
+		if d, ok := decl.Decl.(*ast.GenDecl); ok {
+			return d.Doc
+		}
+	}
+	return nil
+}
+
+func commentFunc(ctx *blockCtx, fn *gogen.Func, decl *ast.FuncDecl) {
 	start := decl.Name.Pos()
 	if ctx.fileLine && start != token.NoPos {
 		if decl.Doc != nil {
@@ -151,6 +167,9 @@ func compileStmt(ctx *blockCtx, stmt ast.Stmt) {
 	case *ast.BlockStmt:
 		ctx.cb.Block()
 		compileStmts(ctx, v.List)
+		if rec := ctx.recorder(); rec != nil {
+			rec.Scope(v, ctx.cb.Scope())
+		}
 		ctx.cb.End()
 		return
 	case *ast.SelectStmt:
@@ -208,6 +227,9 @@ func compileReturnStmt(ctx *blockCtx, expr *ast.ReturnStmt) {
 						ret.Pos(), "cannot use lambda expression as type %v in return statement", rtyp))
 				}
 				compileLambda(ctx, v, sig)
+			case *ast.SliceLit:
+				rtyp := ctx.cb.Func().Type().(*types.Signature).Results().At(i).Type()
+				compileSliceLit(ctx, v, rtyp)
 			default:
 				compileExpr(ctx, ret, inFlags)
 			}
@@ -221,9 +243,53 @@ func compileIncDecStmt(ctx *blockCtx, expr *ast.IncDecStmt) {
 	ctx.cb.IncDec(gotoken.Token(expr.Tok))
 }
 
+// issue #2107:
+//
+//	a <- v
+//	foo.a <- v
+func isAppendable(x ast.Expr) bool {
+	switch x := x.(type) {
+	case *ast.Ident:
+		return true
+	case *ast.SelectorExpr:
+		_, ok := x.X.(*ast.Ident)
+		return ok
+	}
+	return false
+}
+
 func compileSendStmt(ctx *blockCtx, expr *ast.SendStmt) {
-	compileExpr(ctx, expr.Chan)
-	compileExpr(ctx, expr.Value)
+	cb := ctx.cb
+	ch, vals := expr.Chan, expr.Values
+	stk := cb.InternalStack()
+	if isAppendable(ch) { // a <- v1, v2, v3 (issue #2107)
+		compileExpr(ctx, ch)
+		a := stk.Get(-1)
+		t := a.Type.Underlying()
+		if _, ok := t.(*types.Slice); ok { // a = append(a, v1, v2, v3)
+			stk.Pop()
+			compileExprLHS(ctx, ch)
+			cb.Val(ctx.pkg.Builtin().Ref("append"))
+			stk.Push(a)
+			for _, v := range vals {
+				compileExpr(ctx, v)
+			}
+			flags := gogen.InstrFlags(0)
+			if expr.Ellipsis != 0 { // a = append(a, b...)
+				flags |= gogen.InstrFlagEllipsis
+			}
+			cb.CallWith(len(vals)+1, flags, expr).AssignWith(1, 1, expr)
+			return
+		}
+		goto normal
+	}
+	compileExpr(ctx, ch)
+
+normal:
+	if len(vals) != 1 || expr.Ellipsis != 0 {
+		panic(ctx.newCodeError(vals[0].End(), "can't send multiple values to a channel"))
+	}
+	compileExpr(ctx, vals[0])
 	ctx.cb.Send()
 }
 
@@ -272,8 +338,8 @@ func compileAssignStmt(ctx *blockCtx, expr *ast.AssignStmt) {
 	for _, lhs := range expr.Lhs {
 		compileExprLHS(ctx, lhs)
 	}
-	for _, rhs := range expr.Rhs {
-		switch e := rhs.(type) {
+	for i, rhs := range expr.Rhs {
+		switch e := unparen(rhs).(type) {
 		case *ast.LambdaExpr, *ast.LambdaExpr2:
 			if len(expr.Lhs) == 1 && len(expr.Rhs) == 1 {
 				typ := ctx.cb.Get(-1).Type.(interface{ Elem() types.Type }).Elem()
@@ -285,6 +351,18 @@ func compileAssignStmt(ctx *blockCtx, expr *ast.AssignStmt) {
 			} else {
 				panic(ctx.newCodeErrorf(e.Pos(), "lambda unsupport multiple assignment"))
 			}
+		case *ast.SliceLit:
+			var typ types.Type
+			if len(expr.Lhs) == len(expr.Rhs) {
+				typ, _ = gogen.DerefType(ctx.cb.Get(-1 - i).Type)
+			}
+			compileSliceLit(ctx, e, typ)
+		case *ast.CompositeLit:
+			var typ types.Type
+			if len(expr.Lhs) == len(expr.Rhs) {
+				typ, _ = gogen.DerefType(ctx.cb.Get(-1 - i).Type)
+			}
+			compileCompositeLit(ctx, e, typ, false)
 		default:
 			compileExpr(ctx, rhs, inFlags)
 		}
@@ -359,11 +437,19 @@ func compileRangeStmt(ctx *blockCtx, v *ast.RangeStmt) {
 	if pos == 0 {
 		pos = v.For
 	}
-	cb.RangeAssignThen(pos) // TODO: need NewScope for body
+	cb.RangeAssignThen(pos)
 	if len(defineNames) > 0 {
 		defNames(ctx, defineNames, cb.Scope())
 	}
+	if rec := ctx.recorder(); rec != nil {
+		rec.Scope(v, cb.Scope())
+	}
+	cb.VBlock()
 	compileStmts(ctx, v.Body.List)
+	if rec := ctx.recorder(); rec != nil {
+		rec.Scope(v.Body, cb.Scope())
+	}
+	cb.End(v.Body)
 	cb.SetComments(comments, once)
 	setBodyHandler(ctx)
 	cb.End(v)
@@ -394,15 +480,26 @@ func compileForPhraseStmt(ctx *blockCtx, v *ast.ForPhraseStmt) {
 	if len(defineNames) > 0 {
 		defNames(ctx, defineNames, cb.Scope())
 	}
+	if rec := ctx.recorder(); rec != nil {
+		rec.Scope(v, cb.Scope())
+	}
 	if v.Cond != nil {
 		cb.If()
 		compileExpr(ctx, v.Cond)
 		cb.Then()
 		compileStmts(ctx, v.Body.List)
 		cb.SetComments(comments, once)
+		if rec := ctx.recorder(); rec != nil {
+			rec.Scope(v.Body, cb.Scope())
+		}
 		cb.End()
 	} else {
+		cb.VBlock()
 		compileStmts(ctx, v.Body.List)
+		if rec := ctx.recorder(); rec != nil {
+			rec.Scope(v.Body, cb.Scope())
+		}
+		cb.End(v.Body)
 		cb.SetComments(comments, once)
 	}
 	setBodyHandler(ctx)
@@ -507,6 +604,9 @@ func compileForStmt(ctx *blockCtx, v *ast.ForStmt) {
 	cb := ctx.cb
 	comments, once := cb.BackupComments()
 	cb.For(v)
+	if rec := ctx.recorder(); rec != nil {
+		rec.Scope(v, cb.Scope())
+	}
 	if v.Init != nil {
 		compileStmt(ctx, v.Init)
 	}
@@ -516,6 +616,9 @@ func compileForStmt(ctx *blockCtx, v *ast.ForStmt) {
 		cb.None()
 	}
 	cb.Then(v.Body)
+	if rec := ctx.recorder(); rec != nil {
+		rec.Scope(v.Body, cb.Scope())
+	}
 	compileStmts(ctx, v.Body.List)
 	if v.Post != nil {
 		cb.Post()
@@ -539,6 +642,9 @@ func compileIfStmt(ctx *blockCtx, v *ast.IfStmt) {
 		compileStmt(ctx, v.Init)
 	}
 	compileExpr(ctx, v.Cond)
+	if rec := ctx.recorder(); rec != nil {
+		rec.Scope(v, cb.Scope())
+	}
 	cb.Then(v.Body)
 	compileStmts(ctx, v.Body.List)
 	if e := v.Else; e != nil {
@@ -550,6 +656,9 @@ func compileIfStmt(ctx *blockCtx, v *ast.IfStmt) {
 		}
 	}
 	cb.SetComments(comments, once)
+	if rec := ctx.recorder(); rec != nil {
+		rec.Scope(v.Body, cb.Scope())
+	}
 	cb.End(v)
 }
 
@@ -584,6 +693,9 @@ func compileTypeSwitchStmt(ctx *blockCtx, v *ast.TypeSwitchStmt) {
 		panic("TODO: type switch syntax error, please use x.(type)")
 	}
 	cb.TypeSwitch(name, v)
+	if rec := ctx.recorder(); rec != nil {
+		rec.Scope(v, cb.Scope())
+	}
 	if v.Init != nil {
 		compileStmt(ctx, v.Init)
 	}
@@ -596,10 +708,11 @@ func compileTypeSwitchStmt(ctx *blockCtx, v *ast.TypeSwitchStmt) {
 		if !ok {
 			log.Panicln("TODO: compile TypeSwitchStmt failed - case clause expected.")
 		}
+		cb.TypeCase(c)
 		for _, citem := range c.List {
 			compileExpr(ctx, citem)
 			T := cb.Get(-1).Type
-			if tt, ok := T.(*gox.TypeType); ok {
+			if tt, ok := T.(*gogen.TypeType); ok {
 				T = tt.Type()
 			}
 			var haserr bool
@@ -630,9 +743,12 @@ func compileTypeSwitchStmt(ctx *blockCtx, v *ast.TypeSwitchStmt) {
 				firstDefault = c
 			}
 		}
-		cb.TypeCase(len(c.List), c) // TypeCase(0) means default case
+		cb.Then()
 		compileStmts(ctx, c.Body)
 		commentStmt(ctx, stmt)
+		if rec := ctx.recorder(); rec != nil {
+			rec.Scope(c, cb.Scope())
+		}
 		cb.End(c)
 	}
 	cb.SetComments(comments, once)
@@ -663,6 +779,9 @@ func compileSwitchStmt(ctx *blockCtx, v *ast.SwitchStmt) {
 	} else {
 		cb.None() // switch {...}
 	}
+	if rec := ctx.recorder(); rec != nil {
+		rec.Scope(v, cb.Scope())
+	}
 	cb.Then(v.Body)
 	seen := make(valueMap)
 	var firstDefault ast.Stmt
@@ -671,6 +790,7 @@ func compileSwitchStmt(ctx *blockCtx, v *ast.SwitchStmt) {
 		if !ok {
 			log.Panicln("TODO: compile SwitchStmt failed - case clause expected.")
 		}
+		cb.Case(c)
 		for _, citem := range c.List {
 			compileExpr(ctx, citem)
 			v := cb.Get(-1)
@@ -704,13 +824,16 @@ func compileSwitchStmt(ctx *blockCtx, v *ast.SwitchStmt) {
 				firstDefault = c
 			}
 		}
-		cb.Case(len(c.List), c) // Case(0) means default case
+		cb.Then()
 		body, has := hasFallthrough(c.Body)
 		compileStmts(ctx, body)
 		if has {
 			cb.Fallthrough()
 		}
 		commentStmt(ctx, stmt)
+		if rec := ctx.recorder(); rec != nil {
+			rec.Scope(c, cb.Scope())
+		}
 		cb.End(c)
 	}
 	cb.SetComments(comments, once)
@@ -753,14 +876,16 @@ func compileSelectStmt(ctx *blockCtx, v *ast.SelectStmt) {
 		if !ok {
 			log.Panicln("TODO: compile SelectStmt failed - comm clause expected.")
 		}
-		var n int
+		cb.CommCase(c)
 		if c.Comm != nil {
 			compileStmt(ctx, c.Comm)
-			n = 1
 		}
-		cb.CommCase(n, c) // CommCase(0) means default case
+		cb.Then()
 		compileStmts(ctx, c.Body)
 		commentStmt(ctx, stmt)
+		if rec := ctx.recorder(); rec != nil {
+			rec.Scope(c, cb.Scope())
+		}
 		cb.End(c)
 	}
 	cb.SetComments(comments, once)
@@ -791,7 +916,7 @@ func compileBranchStmt(ctx *blockCtx, v *ast.BranchStmt) {
 	}
 }
 
-func getLabel(ctx *blockCtx, label *ast.Ident) *gox.Label {
+func getLabel(ctx *blockCtx, label *ast.Ident) *gogen.Label {
 	if label != nil {
 		if l, ok := ctx.cb.LookupLabel(label.Name); ok {
 			return l
