@@ -17,11 +17,8 @@
 package parser
 
 import (
-	"bytes"
 	"errors"
-	"io"
 	"io/fs"
-	"os"
 	"path"
 	"strings"
 
@@ -30,6 +27,7 @@ import (
 
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/parser/fsx"
+	"github.com/goplus/gop/parser/iox"
 	"github.com/goplus/gop/token"
 )
 
@@ -57,15 +55,15 @@ type FileSystem = fsx.FileSystem
 
 // -----------------------------------------------------------------------------
 
-// Parse parses a single Go+ source file. The target specifies the Go+ source file.
+// Parse parses a single Go+ source file. The filename specifies the Go+ source file.
 // If the file couldn't be read, a nil map and the respective error are returned.
-func Parse(fset *token.FileSet, target string, src interface{}, mode Mode) (pkgs map[string]*ast.Package, err error) {
-	file, err := ParseFile(fset, target, src, mode)
+func Parse(fset *token.FileSet, filename string, src interface{}, mode Mode) (pkgs map[string]*ast.Package, err error) {
+	file, err := ParseFile(fset, filename, src, mode)
 	if err != nil {
 		return
 	}
 	pkgs = make(map[string]*ast.Package)
-	pkgs[file.Name.Name] = astFileToPkg(file, target)
+	pkgs[file.Name.Name] = astFileToPkg(file, filename)
 	return
 }
 
@@ -229,7 +227,7 @@ func ParseFSEntry(fset *token.FileSet, fs FileSystem, filename string, src inter
 
 func filter(d fs.DirEntry, fn func(fs.FileInfo) bool) bool {
 	fi, err := d.Info()
-	return err != nil || fn(fi)
+	return err == nil && fn(fi)
 }
 
 func reqPkg(pkgs map[string]*ast.Package, name string) *ast.Package {
@@ -247,20 +245,22 @@ func reqPkg(pkgs map[string]*ast.Package, name string) *ast.Package {
 func defaultClassKind(fname string) (isProj bool, ok bool) {
 	ext := path.Ext(fname)
 	switch ext {
-	case ".gmx":
-		return true, true
 	case ".spx":
 		return fname == "main.spx", true
+	case ".gsh", ".gmx":
+		return true, true
 	}
 	return
 }
 
 // -----------------------------------------------------------------------------
 
+// ParseFiles parses Go+ source files and returns the corresponding packages.
 func ParseFiles(fset *token.FileSet, files []string, mode Mode) (map[string]*ast.Package, error) {
 	return ParseFSFiles(fset, fsx.Local, files, mode)
 }
 
+// ParseFSFiles parses Go+ source files and returns the corresponding packages.
 func ParseFSFiles(fset *token.FileSet, fs FileSystem, files []string, mode Mode) (map[string]*ast.Package, error) {
 	ret := map[string]*ast.Package{}
 	fabs := (mode & SaveAbsFile) != 0
@@ -286,6 +286,39 @@ func ParseFSFiles(fset *token.FileSet, fs FileSystem, files []string, mode Mode)
 	return ret, nil
 }
 
+// ParseEntries parses Go+ source files and returns the corresponding packages.
+// Compared to ParseFiles, ParseEntries detects fileKind by its filename.
+func ParseEntries(fset *token.FileSet, files []string, conf Config) (map[string]*ast.Package, error) {
+	return ParseFSEntries(fset, fsx.Local, files, conf)
+}
+
+// ParseFSEntries parses Go+ source files and returns the corresponding packages.
+// Compared to ParseFSFiles, ParseFSEntries detects fileKind by its filename.
+func ParseFSEntries(fset *token.FileSet, fs FileSystem, files []string, conf Config) (map[string]*ast.Package, error) {
+	ret := map[string]*ast.Package{}
+	fabs := (conf.Mode & SaveAbsFile) != 0
+	if fabs {
+		conf.Mode &^= SaveAbsFile
+	}
+	for _, file := range files {
+		if fabs {
+			file, _ = fs.Abs(file)
+		}
+		f, err := ParseFSEntry(fset, fs, file, nil, conf)
+		if err != nil {
+			return nil, err
+		}
+		pkgName := f.Name.Name
+		pkg, ok := ret[pkgName]
+		if !ok {
+			pkg = &ast.Package{Name: pkgName, Files: make(map[string]*ast.File)}
+			ret[pkgName] = pkg
+		}
+		pkg.Files[file] = f
+	}
+	return ret, nil
+}
+
 // -----------------------------------------------------------------------------
 
 // ParseFile parses the source code of a single Go+ source file and returns the corresponding ast.File node.
@@ -305,40 +338,9 @@ func ParseFSFile(fset *token.FileSet, fs FileSystem, filename string, src interf
 	return parseFile(fset, filename, code, mode)
 }
 
-var (
-	errInvalidSource = errors.New("invalid source")
-)
-
-func readSource(src interface{}) ([]byte, error) {
-	switch s := src.(type) {
-	case string:
-		return []byte(s), nil
-	case []byte:
-		return s, nil
-	case *bytes.Buffer:
-		// is io.Reader, but src is already available in []byte form
-		if s != nil {
-			return s.Bytes(), nil
-		}
-	case io.Reader:
-		return io.ReadAll(s)
-	}
-	return nil, errInvalidSource
-}
-
-// If src != nil, readSource converts src to a []byte if possible;
-// otherwise it returns an error. If src == nil, readSource returns
-// the result of reading the file specified by filename.
-func readSourceLocal(filename string, src interface{}) ([]byte, error) {
-	if src != nil {
-		return readSource(src)
-	}
-	return os.ReadFile(filename)
-}
-
 func readSourceFS(fs FileSystem, filename string, src interface{}) ([]byte, error) {
 	if src != nil {
-		return readSource(src)
+		return iox.ReadSource(src)
 	}
 	return fs.ReadFile(filename)
 }
