@@ -55,6 +55,7 @@ type Scanner struct {
 	lineOffset int  // current line offset
 	nParen     int
 	insertSemi bool // insert a semicolon before next newline
+	needUnit   bool
 
 	// public state - ok to modify
 	ErrorCount int // number of errors encountered
@@ -509,6 +510,8 @@ func (s *Scanner) scanNumber() (token.Token, string) {
 	} else if s.ch == 'r' {
 		tok = token.RAT
 		s.next()
+	} else if isLetter(s.ch) {
+		s.needUnit = true
 	}
 
 	lit := string(s.src[offs:s.offset])
@@ -828,6 +831,12 @@ scanAgain:
 
 	// determine token value
 	insertSemi := false
+	if s.needUnit { // number with unit
+		insertSemi = true
+		tok, lit = token.UNIT, s.scanIdentifier()
+		s.needUnit = false
+		goto done
+	}
 	switch ch := s.ch; {
 	case isLetter(ch):
 		lit = s.scanIdentifier()
@@ -835,10 +844,17 @@ scanAgain:
 			// keywords are longer than one letter - avoid lookup otherwise
 			tok = token.Lookup(lit)
 			switch tok {
-			case token.IDENT, token.BREAK, token.CONTINUE, token.FALLTHROUGH, token.RETURN:
+			case token.IDENT:
+				insertSemi = true
+				if lit == "py" && s.ch == '"' { // py"..."
+					s.next()
+					tok = token.PYSTRING
+					lit = s.scanString()
+				}
+			case token.BREAK, token.CONTINUE, token.FALLTHROUGH, token.RETURN:
 				insertSemi = true
 			}
-		} else if lit == "C" && s.ch == '"' { // C"..."
+		} else if (lit == "c" || lit == "C") && s.ch == '"' { // c"..."
 			s.next()
 			insertSemi = true
 			tok = token.CSTRING
@@ -918,9 +934,14 @@ scanAgain:
 				insertSemi = true
 			}
 		case '-':
-			tok = s.switch3(token.SUB, token.SUB_ASSIGN, '-', token.DEC)
-			if tok == token.DEC {
-				insertSemi = true
+			if s.ch == '>' { // ->
+				s.next()
+				tok = token.SRARROW
+			} else { // --, -=
+				tok = s.switch3(token.SUB, token.SUB_ASSIGN, '-', token.DEC)
+				if tok == token.DEC {
+					insertSemi = true
+				}
 			}
 		case '*':
 			tok = s.switch2(token.MUL, token.MUL_ASSIGN)
@@ -967,16 +988,19 @@ scanAgain:
 		case '^':
 			tok = s.switch2(token.XOR, token.XOR_ASSIGN)
 		case '<':
-			if s.ch == '-' {
+			if s.ch == '-' { // <-
 				s.next()
 				tok = token.ARROW
-			} else {
+			} else if s.ch == '>' { // <>
+				s.next()
+				tok = token.BIDIARROW
+			} else { // < <= << <<=
 				tok = s.switch4(token.LSS, token.LEQ, '<', token.SHL, token.SHL_ASSIGN)
 			}
 		case '>':
 			tok = s.switch4(token.GTR, token.GEQ, '>', token.SHR, token.SHR_ASSIGN)
 		case '=':
-			tok = s.switch3(token.ASSIGN, token.EQL, '>', token.RARROW)
+			tok = s.switch3(token.ASSIGN, token.EQL, '>', token.DRARROW)
 		case '!':
 			tok = s.switch2(token.NOT, token.NEQ)
 			if tok == token.NOT {
@@ -994,6 +1018,8 @@ scanAgain:
 		case '?':
 			tok = token.QUESTION
 			insertSemi = true
+		case '$':
+			tok = token.ENV
 		default:
 			// next reports unexpected BOMs - don't repeat
 			if ch != bom {
@@ -1004,9 +1030,10 @@ scanAgain:
 			lit = string(ch)
 		}
 	}
+
+done:
 	if s.mode&dontInsertSemis == 0 {
 		s.insertSemi = insertSemi
 	}
-
 	return
 }
